# -*- coding: utf-8 -*-

# CGNES - Coarse-Grained Nuclear Envelope Simulator
# Copyright 2021 Pranjal Singh
#
# See the README file in the top-level CGNES directory.
# This software is released under the GNU General Public License.

# -----------------------------------------------------------------------------
# This file plots MSD-t curve from diffusion tracking of NE components.
#
# Pranjal Singh (orcid.org/0000-0003-3722-2590)
# The Pennsylvania State University
# Contact: pxs392@psu.edu (current); pranjal.officemail@gmail.com (permanent)
# -----------------------------------------------------------------------------

# date             :06/19/2021
# version          :0.8.2
# usage            :gnuplot msd.gp
# gnuplot_version  :5.2.7

set term svg size 500,500 font "Arial,20"
# font "Times-Roman,22"
set output 'msd.svg'
set encoding utf8

#set key outside
# set key box opaque
set key horiz
set key right bottom font ",16"
set key width 10

# Parameters
dNPC = 70 # in nm
dA = 20
dB = 20
dE = 70
dH = 70
kBT = 1.8e-20 # in J
UCx = 2.18/1000.0 # Unit conversion timestep to ns
UCy = 100.0 # sigma^2 to nm^2
# UCD = (1e12)*((10e-9)**2)/(2.18e-9) # sigma^2/tau to μm^2/s
# UCeta = 1000*(1.8e-20)*(2.18e-9)/((10e-9)**3) # epsilon*tau/sigma^3 to cP
UCeta = 1000/(1e-9*(1e-6)**2) # J/(nm*(μm^2/s)) to cP
psv = 0.5 # point size
pt1 = 0 # point type
pt2 = 2
pt3 = 4
pt4 = 6
pt5 = 8
pt6 = 10

# Data
# directly imported in Plot section

# Data bounds
xmin = 0.1
xmax = 30000
xfit = 3000 # fitting linear equation afterwards

# Fitting
DNPC = 1 # in μm^2/s
DA = 7.0
DB = 9.5
DE = 10
DH = 10
f2(x) = 1e-3*(2*2*DNPC*x)
f3(x) = 1e-3*(2*3*DA*x)
f4(x) = 1e-3*(2*3*DB*x)
f5(x) = 1e-3*(2*3*DE*x)
f6(x) = 1e-3*(2*3*DH*x)
# set xrange [xfit:xmax] # fitting range
# fit f2(x) "../data/msdNPC1000.dat" via DNPC
# fit f3(x) "../data/msdA1000.dat" via DA
# fit f4(x) "../data/msdB1000.dat" via DB
# fit f5(x) "../data/msdE1000.dat" via DE
# fit f6(x) "../data/msdH1000.dat" via DH

# Fitting output
print "D in μm^2/s"
print "DNPC = ",DNPC," DA = ",DA," DB = ",DB," DE = ",DE," DH = ",DH
etaNPC = UCeta*(kBT/(3*3.1416*DNPC*dNPC))
etaA = UCeta*(kBT/(3*3.1416*DA*dA))
etaB = UCeta*(kBT/(3*3.1416*DB*dB))
etaE = UCeta*(kBT/(3*3.1416*DE*dE))
etaH = UCeta*(kBT/(3*3.1416*DH*dH))
print "eta in cP"
print "etaNPC = ",etaNPC," etaA = ",etaA," etaB = ",etaB," etaE = ",etaE," etaH = ",etaH

# Axes
set xrange [xmin:xmax]
set yrange [0.001:1000]
set format x "10^{%L}"
set format y "10^{%L}"
set logscale x 10
set logscale y 10
set xtics 10
set ylabel 'Mean-squared displacement, <r^{2}> (nm^2)'
set xlabel 'Time, t (ns)'

# Plot
plot "../data/msdNPC0.dat" u ($1*UCx):($2*UCy) pt pt2 ps psv lc rgb '#0096c8' title 'NPC', \
     "../data/msdNPC1.dat" u ($1*UCx):($2*UCy) pt pt2 ps psv lc rgb '#0096c8' notitle, \
     "../data/msdNPC10.dat" u ($1*UCx):($2*UCy) pt pt2 ps psv lc rgb '#0096c8' notitle, \
     "../data/msdNPC100.dat" u ($1*UCx):($2*UCy) pt pt2 ps psv lc rgb '#0096c8' notitle, \
     "../data/msdNPC1000.dat" u ($1*UCx):($2*UCy) pt pt2 ps psv lc rgb '#0096c8' notitle, \
     "../data/msdNLA0.dat" u ($1*UCx):($2*UCy) pt pt3 ps psv lc rgb '#00af00' title 'NL-A', \
     "../data/msdNLA1.dat" u ($1*UCx):($2*UCy) pt pt3 ps psv lc rgb '#00af00' notitle, \
     "../data/msdNLA10.dat" u ($1*UCx):($2*UCy) pt pt3 ps psv lc rgb '#00af00' notitle, \
     "../data/msdNLA100.dat" u ($1*UCx):($2*UCy) pt pt3 ps psv lc rgb '#00af00' notitle, \
     "../data/msdNLA1000.dat" u ($1*UCx):($2*UCy) pt pt3 ps psv lc rgb '#00af00' notitle, \
     "../data/msdNLB0.dat" u ($1*UCx):($2*UCy) pt pt4 ps psv lc rgb '#c80000' title 'NL-B', \
     "../data/msdNLB1.dat" u ($1*UCx):($2*UCy) pt pt4 ps psv lc rgb '#c80000' notitle, \
     "../data/msdNLB10.dat" u ($1*UCx):($2*UCy) pt pt4 ps psv lc rgb '#c80000' notitle, \
     "../data/msdNLB100.dat" u ($1*UCx):($2*UCy) pt pt4 ps psv lc rgb '#c80000' notitle, \
     "../data/msdNLB1000.dat" u ($1*UCx):($2*UCy) pt pt4 ps psv lc rgb '#c80000' notitle, \
     "../data/msdEC0.dat" u ($1*UCx):($2*UCy) pt pt5 ps psv lc rgb '#000000' title 'EC', \
     "../data/msdEC1.dat" u ($1*UCx):($2*UCy) pt pt5 ps psv lc rgb '#000000' notitle, \
     "../data/msdEC10.dat" u ($1*UCx):($2*UCy) pt pt5 ps psv lc rgb '#000000' notitle, \
     "../data/msdEC100.dat" u ($1*UCx):($2*UCy) pt pt5 ps psv lc rgb '#000000' notitle, \
     "../data/msdEC1000.dat" u ($1*UCx):($2*UCy) pt pt5 ps psv lc rgb '#000000' notitle, \
     "../data/msdHC0.dat" u ($1*UCx):($2*UCy) pt pt6 ps psv lc rgb '#0000af' title 'HC', \
     "../data/msdHC1.dat" u ($1*UCx):($2*UCy) pt pt6 ps psv lc rgb '#0000af' notitle, \
     "../data/msdHC10.dat" u ($1*UCx):($2*UCy) pt pt6 ps psv lc rgb '#0000af' notitle, \
     "../data/msdHC100.dat" u ($1*UCx):($2*UCy) pt pt6 ps psv lc rgb '#0000af' notitle, \
     "../data/msdHC1000.dat" u ($1*UCx):($2*UCy) pt pt6 ps psv lc rgb '#0000af' notitle, \
     [xfit:] f2(x) lw 2 lc rgb 'cyan' t 'SER fit', \
     [xfit:] f3(x) lw 2 lc rgb 'green' notitle, \
     [xfit:] f4(x) lw 2 lc rgb 'red' notitle, \
     [xfit:] f5(x) lw 2 lc rgb 'black' notitle, \
     [xfit:] f6(x) lw 2 lc rgb 'blue' notitle


quit
