#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# CGNES - Coarse-Grained Nuclear Envelope Simulator
# Copyright 2021 Pranjal Singh
#
# See the README file in the top-level CGNES directory.
# This software is released under the GNU General Public License.

# -----------------------------------------------------------------------------
# This file (ne.py) generates configuration data file (ne.data)
# which will be read by init.lmp.
#
# Pranjal Singh (orcid.org/0000-0003-3722-2590)
# The Pennsylvania State University
# Contact: pxs392@psu.edu (current); pranjal.officemail@gmail.com (permanent)
# -----------------------------------------------------------------------------

# date         :06/19/2021
# version      :0.8.2
# usage        :python ne.py
# py_version   :3.7

# Initialize ==================================================================

import sys
import numpy as np
import random

scl = float(sys.argv[1]) # scale of simulation box
L = 100*scl # Length of sides of simulation box in x and y (Lx = Ly = L)
sd = 221 # seed for random generator

mNM = 1.024
mNPC = 55.479
mNLA = 0.064
mNLB = 0.063

if scl == 2.5:
    scl1 = 1.022

# 1.055

if scl == 2.5:
    scl2 = 1.0

sclsq = scl*scl

# Simulation box
xlo = -L/2.0
xhi = L/2.0
ylo = -L/2.0
yhi = L/2.0
zlo = -300.0
zhi = 150.0

# Initialize counts
natoms = 0
nbonds = 0
nangles = 0
nellipsoids = 0
natomTs = 0
nbondTs = 0
nangleTs = 0
nmols = 0

# Functions ===================================================================

def monolayer(S,d,H):
    """A 2D triangular lattice of size SxS with points spaced by d.
    """

    xyz = np.zeros(3)

    trimaker = np.asarray([[1.0, np.cos(np.pi/3.0), 0.0],
                           [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]])
    Sx = S*(1.0+np.tan(np.pi/3.0))
    sx = d
    Sy = S
    sy = d*np.sin(np.pi/3.0)

    for i in range(int(Sx/sx)):
        for j in range(int(Sy/sy)):
            x0 = i*sx-Sx/2.0
            y0 = j*sy-Sy/2.0
            z0 = H
            xyz0 = trimaker.dot([x0, y0, z0])
            if np.abs(xyz0[0])<S/2.0:
                xyz = np.vstack([xyz, xyz0])

    xyz = xyz[1:]

    return xyz

def distrib(Nb,NBf,NAf):
    """Filament distribution.
    """

    Bf = []
    Af = []
    for i in range(len(Nb)):
        Bf = np.concatenate((Bf, Nb[i]*np.ones(NBf[i])))
        Af = np.concatenate((Af, Nb[i]*np.ones(NAf[i])))

    nfil = np.sum(NBf)+np.sum(NAf)
    orderedtypes = np.concatenate((3*np.ones(np.sum(NAf)),
                                   4*np.ones(np.sum(NBf))))
    orderedNbs = np.concatenate((Af, Bf))
    p = np.random.RandomState(seed=sd).permutation(nfil)
    filtype = orderedtypes[p].astype(int)
    filNb = orderedNbs[p].astype(int)

    return filtype, filNb

def filaments(S,d,H,filtype,filNb,r0,natoms,nbonds,nangles,nmols):
    """Filaments grown on 2D triangular lattice of size SxS
    with filaments spaced by d and beads spaced by r0.
    """
    atoms = np.zeros(9)
    bonds = np.zeros(4)
    angles = np.zeros(5)

    trimaker = np.asarray([[1.0, np.cos(np.pi/3.0), 0.0],
                           [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]])
    Sx = S*(1.0+np.tan(np.pi/3.0))
    sx = d
    Sy = S
    sy = d*np.sin(np.pi/3.0)

    nfil = len(filtype)
    atom_id = natoms
    bond_id = nbonds
    angle_id = nangles
    fil = 0
    for i in range(int(Sx/sx)):
        for j in range(int(Sy/sy)):
            x0 = i*sx-Sx/2.0
            y0 = j*sy-Sy/2.0
            z0 = H
            xyz0 = trimaker.dot([x0, y0, z0])
            if np.abs(xyz0[0])<S/2.0:
                if fil < nfil:
                    fil += 1
                    seq=0
                    Nb = filNb[fil-1]
                    for k in range(Nb):
                        seq += 1
                        atom_id += 1

                        atomtype = filtype[fil-1]

                        if filtype[fil-1] == 3:
                            btype = 1
                            atommass = mNLA
                            displc = 0.0
                        if filtype[fil-1] == 4:
                            btype = 2
                            atommass = mNLB
                            displc = 0.0

                        atype = 1

                        atoms0 = [atom_id, atomtype, xyz0[0], xyz0[1],
                                  xyz0[2]-r0*k-displc, 0, atommass,
                                  nmols+fil, k]
                        atoms = np.vstack([atoms, atoms0])

                        if seq<Nb:
                            bond_id += 1
                            bonds0 = [bond_id, btype, atom_id, atom_id+1]
                            bonds = np.vstack([bonds, bonds0])

                        if seq>1 and seq<Nb:
                            angle_id += 1
                            angles0 = [angle_id, atype,
                                       atom_id-1, atom_id, atom_id+1]
                            angles = np.vstack([angles, angles0])

    atoms = atoms[1:]
    bonds = bonds[1:]
    angles = angles[1:]

    return atoms, bonds, angles, fil

# Molecules ===================================================================

# NM --------------------------------------------------------------------------
d0 = 2.0
d1 = d0*scl1
h = 0.0

atoms0 = monolayer(L,d1,h)
natoms0 = np.shape(atoms0)[0]

nNPC = np.int(6*sclsq)
listNPC = np.ones(nNPC)
nNM = np.int(natoms0-nNPC)
listNM = np.ones(nNM)
orderedtypes = np.concatenate((2*listNPC, 1*listNM))
orderedmasses = np.concatenate((mNPC*listNPC, mNM*listNM))
p = np.random.RandomState(seed=sd).permutation(natoms0)
types0 = orderedtypes[p].astype(int)
mass = orderedmasses[p]
shape0 = [d0, d0, d0]
volm = (4.0/3.0*(np.pi*shape0[0]*shape0[1]*shape0[2])/8.0)
density0 = mass/volm
ellipsoidflag0 = 1
molecule_ID0 = 0
charge0 = 0

nquat=[0, -1, 0]
thetaquat=np.pi/2.0
quat = [np.cos(thetaquat/2.0),nquat[0]*np.sin(thetaquat/2.0),
        nquat[1]*np.sin(thetaquat/2.0),nquat[2]*np.sin(thetaquat/2.0)]

natoms += natoms0
nellipsoids += natoms0
natomTs += 2

print(f'{natoms} NM particles generated.')

# NL --------------------------------------------------------------------------
d1 = 7.335*scl2
h = -5.0
Nb  = [9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 43]
NBf = np.asarray([3, 12, 16, 11, 11, 7, 4, 2, 1, 0, 1, 0])*sclsq
NAf = np.asarray([7, 24, 33, 21, 23, 15, 8, 3, 3, 1, 0, 1])*sclsq
NAf = np.rint(NAf).astype(np.int)
NBf = np.rint(NBf).astype(np.int)

r0 = 2.0

(filtype, filNb) = distrib(Nb,NBf,NAf)
(atoms1, bonds1, angles1, fil) = filaments(L-2.0,d1,h,filtype,filNb,r0,
                                      natoms,nbonds,nangles,nmols)

natoms1 = np.shape(atoms1)[0]
nbonds1 = np.shape(bonds1)[0]
nangles1 = np.shape(angles1)[0]

natoms += natoms1
nbonds += nbonds1
nangles += nangles1
natomTs += 2
nbondTs += 4
nangleTs += 1
nfil = len(filtype)
nmols += fil

print(f'{fil} of {nfil} NL filaments generated.')

fracf = np.around(100*(NAf+NBf)/np.sum(NAf+NBf),2)
print(f'NL filament distribution: {fracf}.')


natoms += 0
nbonds += 0
nangles += 0
natomTs += 2
nbondTs += 1
nangleTs += 0
nmols += 0

# Write =======================================================================

with open('ne.data','w') as fout:
    # Title comment -----------------------------------------------------------
    fout.write('Initial configuration of NE\n\n')

    # Header ------------------------------------------------------------------
    fout.write(f'{natoms} atoms\n')
    fout.write(f'{natomTs} atom types\n')
    fout.write(f'{nbonds} bonds\n')
    fout.write(f'{nbondTs} bond types\n')
    fout.write(f'{nangles} angles\n')
    fout.write(f'{nangleTs} angle types\n')
    fout.write(f'{nellipsoids} ellipsoids\n')
    fout.write('\n')

    # Box dimensions ----------------------------------------------------------
    fout.write('{} {} xlo xhi\n'.format(xlo, xhi))
    fout.write('{} {} ylo yhi\n'.format(ylo, yhi))
    fout.write('{} {} zlo zhi\n'.format(zlo, zhi))
    fout.write('\n')

    # Atoms section -----------------------------------------------------------
    fout.write('Atoms # hybrid\n\n')

    jadd = 1
    for j in range(natoms0):
        fout.write('{:.0f} {:.0f} {:.6f} {:.6f} {:.6f}'\
                   ' {:.0f} {:.4f} {:.0f} {:.0f} 0 0 0'\
                   '\n'.format(j+jadd,types0[j],*tuple(atoms0[j,:]),
                   ellipsoidflag0,density0[j],molecule_ID0,charge0))

    jadd += natoms0
    for j in range(natoms1):
        fout.write('{:.0f} {:.0f} {:.6f} {:.6f} {:.6f}'\
                   ' {:.0f} {:.4f} {:.0f} {:.0f} 0 0 0'\
                   '\n'.format(*tuple(atoms1[j,:])))

    fout.write('\n')

    # Bonds section -----------------------------------------------------------
    fout.write('Bonds\n\n')

    jadd = 1
    for j in range(nbonds1):
        fout.write('{:.0f} {:.0f} {:.0f} {:.0f}'\
                   '\n'.format(*tuple(bonds1[j,:])))

    fout.write('\n')

    # Angles section ----------------------------------------------------------
    fout.write('Angles\n\n')

    jadd = 1
    for j in range(nangles1):
        fout.write('{:.0f} {:.0f} {:.0f} {:.0f} {:.0f}'\
                   '\n'.format(*tuple(angles1[j,:])))

    fout.write('\n')

    # Ellipsoids section ------------------------------------------------------
    fout.write('Ellipsoids\n\n');

    jadd = 1
    for j in range(natoms0):
            fout.write('{:.0f} {:.2f} {:.2f} {:.2f} {:.6f} '\
                       '{:.6f} {:.6f} {:.6f}\n'.format(j+jadd,
                       *tuple(shape0),*tuple(quat[:])))

    fout.write('\n')

print('ne.data generated.')

# Exit ========================================================================
sys.exit()
